@@ -1,9 +1,16 @@
 #!/usr/bin/env python3
 # Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
 
+from .dp_ddp import DifferentiallyPrivateDistributedDataParallel
 from .dp_lstm import DPLSTM
 from .dp_multihead_attention import DPMultiheadAttention, SequenceBias
 from .param_rename import ParamRenamedModule
 
 
-__all__ = ["DPLSTM", "DPMultiheadAttention", "ParamRenamedModule", "SequenceBias"]
+__all__ = [
+    "DPLSTM",
+    "DPMultiheadAttention",
+    "ParamRenamedModule",
+    "SequenceBias",
+    "DifferentiallyPrivateDistributedDataParallel",
+]
