@@ -1,3 +1,6 @@
+#!/usr/bin/env python3
+# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
+
 import math
 from typing import List, Optional, Tuple, Union
 
@@ -138,6 +141,13 @@ class DPLSTMCell(nn.Module):
         for weight in self.parameters():
             nn.init.uniform_(weight, -stdv, stdv)
 
+    def set_max_batch_length(self, max_batch_length: int) -> None:
+        """
+        Sets max batch length
+        """
+        self.ih.max_batch_len = max_batch_length
+        self.hh.max_batch_len = max_batch_length
+
     def forward(
         self,
         x: torch.Tensor,
@@ -145,7 +155,6 @@ class DPLSTMCell(nn.Module):
         c_prev: torch.Tensor,
         batch_size_t: Optional[int] = None,
     ) -> Tuple[torch.Tensor, torch.Tensor]:
-
         if batch_size_t is None:
             gates = self.ih(x) + self.hh(h_prev)  # [B, 4*D]
         else:
@@ -205,6 +214,12 @@ class DPLSTMLayer(nn.Module):
 
         self.dropout_layer = nn.Dropout(dropout) if dropout > 0 else None
 
+    def set_max_batch_length(self, max_batch_length: int) -> None:
+        """
+        Sets max batch length. Useful for PackedSequences
+        """
+        self.cell.set_max_batch_length(max_batch_length)
+
     def forward(
         self,
         x: Union[torch.Tensor, Tuple],
@@ -321,6 +336,13 @@ class BidirectionalDPLSTMLayer(nn.Module):
             reverse=True,
         )
 
+    def set_max_batch_length(self, max_batch_length: int) -> None:
+        """
+        Sets max batch length
+        """
+        self.forward_layer.set_max_batch_length(max_batch_length)
+        self.reverse_layer.set_max_batch_length(max_batch_length)
+
     def forward(
         self,
         x: torch.Tensor,
@@ -455,6 +477,8 @@ class DPLSTM(ParamRenamedModule):
             B = batch_sizes[0].item()
             _, D = x.shape
             x = x.split(tuple(batch_sizes))
+            for layer in self.layers:
+                layer.set_max_batch_length(B)
         else:
             sorted_indices = None
             unsorted_indices = None
