@@ -272,15 +272,17 @@ def get_privacy_spent(
 ) -> Tuple[float, float]:
     r"""Computes epsilon given a list of Renyi Differential Privacy (RDP) values at
     multiple RDP orders and target ``delta``.
-
+    The computation of epslion, i.e. conversion from RDP to (eps, delta)-DP,
+    is based on the theorem presented in the following work:
+    Borja Balle et al. "Hypothesis testing interpretations and Renyi differential privacy."
+    International Conference on Artificial Intelligence and Statistics. PMLR, 2020.
+    Particullary, Theorem 21 in the arXiv version https://arxiv.org/abs/1905.09982.
     Args:
         orders: An array (or a scalar) of orders (alphas).
         rdp: A list (or a scalar) of RDP guarantees.
         delta: The target delta.
-
     Returns:
         Pair of epsilon and optimal order alpha.
-
     Raises:
         ValueError
             If the lengths of ``orders`` and ``rdp`` are not equal.
@@ -295,7 +297,11 @@ def get_privacy_spent(
             f"\trdp_vec = {rdp_vec}\n"
         )
 
-    eps = rdp_vec - math.log(delta) / (orders_vec - 1)
+    eps = (
+        rdp_vec
+        - (np.log(delta) + np.log(orders_vec)) / (orders_vec - 1)
+        + np.log((orders_vec - 1) / orders_vec)
+    )
 
     # special case when there is no privacy
     if np.isnan(eps).all():
