@@ -133,7 +133,7 @@ def nullify_batchnorm_modules(root: nn.Module) -> nn.Module:
         will heavily affect convergence of the model.
     """
     return replace_all_modules(
-        root, nn.modules.batchnorm._BatchNorm, lambda _: nn.Identity()  # pyre-ignore
+        root, nn.modules.batchnorm._BatchNorm, lambda _: nn.Identity()
     )
 
 
@@ -167,6 +167,4 @@ def convert_batchnorm_modules(
         >>>  print(model.layer1[0].bn1)
         GroupNorm module details
     """
-    # pyre-fixme[6]: Expected `Type[nn.Module]` for 2nd param but got
-    #  `Type[nn.modules.batchnorm._BatchNorm]`.
     return replace_all_modules(model, nn.modules.batchnorm._BatchNorm, converter)
